
/*
This is a simple demonstration of some of the F001 routines. 

The demonstration configures the B421 as GPIB system controller
and determines the addresses at which there are listening devices.

If you do not have any GPIB devices, or even a GPIB bus cable to
connect to the B421 TRAM, you can still run this code; it will simply
indicate that it could find no devices on the bus.

Please refer to the F001 user manual for a complete description of 
the commands used in this demonstration
*/

#include <misc.h>
#include <stdio.h>
#include <stdlib.h>
#include <process.h>
#include <channel.h>

/* The next header file, f001io.h, provides the prototypes for the F001IO 
   procedures. It should be included in all files which refer to F001IO
   routines */
   
#include <f001io.h>

/* f001def.h provides various constant definitions for error codes
   and general use. You should include it if you wish to use the
   predefined constants provided therein. The f001def.h file should
   be examined for further information */

#include <f001def.h>

#define DEMO_STACK               0xE00
#define TRUE                         1
#define FALSE                        0
#define MIN_PRIMARY_ADDRESS          0
#define MAX_PRIMARY_ADDRESS         30
#define LISTENER_PRESENT          TRUE
#define LISTENER_NOT_PRESENT     FALSE
#define NO_OF_PRIMARY_ADDRESSES     31

/*
The next function is the demonstration process. It configures the B421
as GPIB system controller and determines the addresses at which there
are listening devices.

If you do not have any GPIB devices, or even a GPIB cable to connect to
the B421 TRAM, you can still run this code; it will simply indicate
that it could find no devices on the bus.

Please refer to the B421 user manual for a complete demonstration of the
commands used in this demonstration.
*/

void demo_proc (Process *p, Channel *from_f001, Channel *to_f001)

  {
   #define ADDRESS_SIZE 1
   #define BUFFER_SIZE  1

   short error, source, gpib_addr, device_mode, driver_type, tx_period,
         term, i, count;

   char term_char;
   char buffer [BUFFER_SIZE];
    
   short listeners [NO_OF_PRIMARY_ADDRESSES];
   short address [ADDRESS_SIZE];

   /*
   The next statement prevents the compiler generating spurious error
   messages about unused variables
   */
   p = p;

   /*
   first initialise the B421 TRAM - this is mandatory; no commands can be 
   invoked until all the user definable TRAM features have been selected.
   */
   f001_start_setup (from_f001, to_f001, &error);
   printf ("f001_start_setup error : %d\n", error);

   source = PARAMETER;
   gpib_addr = 3;
   f001_set_gpib_address (from_f001, to_f001, source, gpib_addr, &error);
   printf ("f001_set_gpib_address error : %d\n", error);

   device_mode = SYSTEM_CONTROLLER; 
   f001_set_device_mode (from_f001, to_f001, source, device_mode, &error);
   printf ("f001_set_device_mode error : %d\n", error);

   driver_type = TRISTATE;
   f001_set_bus_drivers (from_f001, to_f001, source, driver_type, &error);
   printf ("f001_set_bus_drivers error : %d\n", error);

   tx_period = 100; /* ms */
   f001_set_timeout (from_f001, to_f001, tx_period, &error);
   printf ("f001_set_timeout error : %d\n", error);

   term = EOI_TERM;
   f001_set_tx_terminator (from_f001, to_f001, term, &error);
   printf ("f001_set_tx_terminator error : %d\n", error);

   term = LF_OR_EOI_TERM;
   f001_set_rx_terminator (from_f001, to_f001, term, term_char, &error);
   printf ("f001_set_rx_terminator error : %d\n", error);

   f001_end_setup (from_f001, to_f001, &error);
   printf ("f001_end_setup error : %d\n", error);

   /*
   if there was an error during initialisation return to DOS
   */
   
   if (error != NO_ERROR)
     {
      printf ("\n The B421 did not initialise correctly\n");
      printf ("B421 initialisation error : %d\n", error);
      exit_terminate (EXIT_FAILURE);
     }
     
   /*
   pulse IFC true for 128 microseconds - this causes all devices on the 
   bus to terminate any transaction in progress
   */   

   f001_send_ifc (from_f001, to_f001, &error);
   printf ("f001_send_ifc error : %d\n", error);
   
   /*
   set the REN line true, so all devices enter a remote state
   */

   f001_set_ren (from_f001, to_f001, FALSE, &error);
   
   /*
   all devices on the bus should now be in an idle state and 
   listening for commands, so determine where they are.
   First initialise some variables.
   */
   
   buffer [0] = 0;
   count = 1;
   for (i = MIN_PRIMARY_ADDRESS; i <= MAX_PRIMARY_ADDRESS; i++)
     listeners [i] = LISTENER_NOT_PRESENT;

   /*
   Now check that there is at least one device on the bus by addressing
   device #1 to listen; if there is not, exit immediately.
   */

   address [0] = 1;
   f001_send_setup (from_f001, to_f001, address, ADDRESS_SIZE, &error);
   if (error == BYTE_TRANSMIT_TIMED_OUT)
     {
      /* then there are no devices on the bus so exit */
      
      printf ("\n There are no devices on the bus\n");
      exit_terminate (EXIT_FAILURE);
     }
   else if ((error != NO_ERROR) && (error != BYTE_TRANSMIT_TIMED_OUT))
     {
      /* then some other error occurred so report it and exit */

      printf ("\n F001 error : %d\n", error);
      exit_terminate (EXIT_FAILURE);
     }
 
   /* there is at least one device on the bus, so find out where 
      they are 
   */

   for (i = MIN_PRIMARY_ADDRESS; i <= MAX_PRIMARY_ADDRESS; i++)
     {
      address [0] = i;
      f001_send (from_f001, to_f001, address, ADDRESS_SIZE, count, 
                 buffer, BUFFER_SIZE, &error);
      if (error == NO_ERROR)
        {
         /* then there is a listener at the current address */
         
         listeners [i] = LISTENER_PRESENT;
        }
      else if (error == BYTE_TRANSMIT_TIMED_OUT ) 
        {
         /* then there is no listener at this address so do nothing */ ;
        }
      else if ((error != NO_ERROR) && (error != BYTE_TRANSMIT_TIMED_OUT))
        {
         /* then some other occurred so report it and exit */

         printf ("\n F001 error : %d\n", error);
         exit_terminate (EXIT_FAILURE);
        }
     }
 
   /* now report on the locations of the listeners */
   printf ("\n There are listeners at the following addresses : \n");
   for (i = MIN_PRIMARY_ADDRESS; i <= MAX_PRIMARY_ADDRESS; i++)
     {
      if (listeners [i] == LISTENER_PRESENT)
        printf (" %d ", i);
     }     

   /* we are now finished so exit */

    exit_terminate (EXIT_FAILURE);
  }

int main (void)

  {

   /*
   The code that follows configures the demo_proc function as a parallel 
   process. This process will run on an IMS B404 TRAM or similar; i.e.
   a T8 based TRAM with 2M of RAM.

   As C does not support the occam channel or process concepts as part 
   of the language definition, Inmos provides a run-time library that
   allows the user to define parallel processes and channels within a
   C program. It is beyond the scope of this example to describe these
   facilities in detail, and the reader should consult the C toolset
   documentation for further information. The comments in the code that
   follows, briefly explain the procedure for writing parallel code in C
   and indicates how the same effect could be achieved in occam. The 
   reader who is unfamiliar with occam should ignore any allusion to that
   language.
   */

   /* 
   The next line declares the channel variables used for communication
   between the demo_proc and f001_proc function. The type Channel is
   defined in the channel.h header file. Note that from_f001 and to_f001
   are pointers to Channel.

   The occam equivalent to the next line is :

   CHAN OF F001PROT from_f001, to_f001:   
   */

   Channel *from_f001, *to_f001;

   /*
   Now we declare the Process variables that are used later to start
   the demo_proc as a parallel processe. The type Process is defined 
   in the process.h header file. Again, note that demop is a pointer to 
   Process.
 
   There is no exact occam equivalent to this declaration; the occam
   PAR statement combines the action of both the next line and the call 
   to the ProcPar function below.
   */

   Process *demop;

   short no_of_params;

   /*
   Having declared the from_f001 and to_f001 channels, we now have to
   determine their link addresses, which are defined in the associated
   twotrams.cfs file. This is done via the get_param function which reads
   parameters from the list given in the interface attribute for a process
   in the .cfs file.   

   There is no exact occam equivalent to the piece of code that follows.
   */

   from_f001 = (Channel *) get_param (3);
   if (from_f001 == NULL)
     {
      printf ("Unable to obtain a valid channel pointer for from_f001 \n");
      exit_terminate (EXIT_FAILURE);
     }

   to_f001 = (Channel *) get_param (4);
   if (to_f001 == NULL)
     {
      printf ("Unable to obtain a valid channel pointer for to_f001 \n");
      exit_terminate (EXIT_FAILURE);
     }

   /*
   Now we allocate memory for the demo_proc process using the ProcAlloc 
   function. In occam, this step is performed at compile time so again 
   there is no occam equivalent. 

   Note that the program is aborted if there is insufficient memory for 
   the process.
   */
    
   no_of_params = 2;
   demop = ProcAlloc (demo_proc, DEMO_STACK, no_of_params, 
                      from_f001, to_f001);
   if (demop == NULL)
     {
      printf ("Unable to allocate demop process space\n");
      exit_terminate (EXIT_FAILURE);
     }

   /*
   We now start demo_proc as a parallel process via the ProcPar function.
   This is equivalent to the occam statement :

   PAR
     demo_proc ()

   */
      
   ProcPar (demop, NULL);

  }
